class CorpusList extends RessourceList {
  static options = {
    item: `
      <tr class="hoverable">
        <td><a class="btn-floating disabled"><i class="material-icons nopaque-service-color darken" data-service="corpus-analysis">book</i></a></td>
        <td><b class="title"></b><br><i class="description"></i></td>
        <td><span class="status badge new corpus-status-color corpus-status-text" data-badge-caption=""></span></td>
        <td class="right-align">
          <a class="action-button btn-floating red tooltipped waves-effect waves-light" data-action="delete" data-position="top" data-tooltip="Delete"><i class="material-icons">delete</i></a>
          <a class="action-button btn-floating nopaque-service-color darken tooltipped waves-effect waves-light" data-action="view" data-position="top" data-service="corpus-analysis" data-tooltip="View"><i class="material-icons">send</i></a>
        </td>
      </tr>
    `.trim(),
    ressourceMapper: corpus => {
      return {
        'id': corpus.id,
        'creation-date': corpus.creation_date,
        'description': corpus.description,
        'status': corpus.status,
        'title': corpus.title
      };
    },
    sortValueName: 'creation-date',
    valueNames: [
      {data: ['id']},
      {data: ['creation-date']},
      {name: 'status', attr: 'data-corpus-status'},
      'description',
      'title'
    ]
  };


  constructor(listElement, options = {}) {
    super(listElement, {...CorpusList.options, ...options});
  }

  init(user) {
    super._init(user.corpora);
  }

  onclick(event) {
    let action;
    let actionButtonElement;
    let corpusElement;
    let corpusId;
    let deleteModal;
    let deleteModalElement;
    let tmp;

    corpusElement = event.target.closest('tr[data-id]');
    if (corpusElement === null) {return;}
    corpusId = corpusElement.dataset.id;
    actionButtonElement = event.target.closest('.action-button[data-action]');
    action = actionButtonElement === null ? 'view' : actionButtonElement.dataset.action;
    switch (action) {
      case 'delete':
        tmp = document.createElement('div');
        tmp.innerHTML = `
          <div class="modal">
            <div class="modal-content">
              <h4>Confirm corpus deletion</h4>
              <p>Do you really want to delete the corpus <b>${app.users[this.userId].corpora[corpusId].title}</b>? All files will be permanently deleted!</p>
            </div>
            <div class="modal-footer">
              <a href="#!" class="btn modal-close waves-effect waves-light">Cancel</a>
              <a class="btn modal-close red waves-effect waves-light" href="/corpora/${corpusId}/delete"><i class="material-icons left">delete</i>Delete</a>
            </div>
          </div>
        `.trim();
        deleteModalElement = document.querySelector('#modals').appendChild(tmp.firstChild);
        deleteModal = M.Modal.init(
          deleteModalElement,
          {
            onCloseEnd: () => {
              deleteModal.destroy();
              deleteModalElement.remove();
            }
          }
        );
        deleteModal.open();
        break;
      case 'view':
        window.location.href = `/corpora/${corpusId}`;
        break;
      default:
        break;
    }
  }

  usersPatchHandler(patch) {
    let corpusId;
    let filteredPatch;
    let match;
    let operation;
    let re;
    let valueName;

    re = new RegExp(`^/users/${this.userId}/corpora/([A-Za-z0-9]*)`);
    filteredPatch = patch.filter(operation => re.test(operation.path));
    for (operation of filteredPatch) {
      switch(operation.op) {
        case 'add':
          re = new RegExp(`^/users/${this.userId}/corpora/([A-Za-z0-9]*)$`);
          if (re.test(operation.path)) {this.add(operation.value);}
          break;
        case 'remove':
          re = new RegExp(`^/users/${this.userId}/corpora/([A-Za-z0-9]*)$`);
          if (re.test(operation.path)) {
            [match, corpusId] = operation.path.match(re);
            this.remove(corpusId);
          }
          break;
        case 'replace':
          re = new RegExp(`^/users/${this.userId}/corpora/([A-Za-z0-9]*)/(status|description|title)$`);
          if (re.test(operation.path)) {
            [match, corpusId, valueName] = operation.path.match(re);
            this.replace(corpusId, valueName, operation.value);
          }
          break;
        default:
          break;
      }
    }
  }
}
